package com.mashangban.example.tool;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.mashangban.example.utils.HttpClientFactory;

public class OpenApiInvoke {
	
	private static final Logger logger = LoggerFactory.getLogger(OpenApiInvoke.class);
	
	private static final String DEFAULT_ENCODING = "UTF-8";
	
	//TODO马上办oauth的地址，根据实际对接调整即可
	public static final String OAUTH_HOST = "https://oauth.qiye.yixin.im";
	//TODO企业自行调整跳转回的url地址
	public static final String RETURN_URL = "https://oa.com/msb/login";
	
	//TODO马上办开放平台的地址
	private static final String OPEN_HOST = "https://open.qiye.yixin.im";
	//TODO请修改为正确的企业应用appkey
	public static final String appKey = "xxxxxxxxxxxx";
	//TODO请修改为正确的企业应用appSecret
	public static final String appSecret = "xxxxxxxxxxxxx";
	//TODO请修改为正确的企业应用permAuth
	public static final String permAuth = "xxxxxxxxxxxxx";
	
	
	/**
	 * URL 编码, Encode默认为UTF-8. 
	 */
	public static String urlEncode(String part) {
		try {
			return URLEncoder.encode(part, DEFAULT_ENCODING);
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * URL 解码, Encode默认为UTF-8. 
	 */
	public static String urlDecode(String part) {

		try {
			return URLDecoder.decode(part, DEFAULT_ENCODING);
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
	}
	
	public static String getAccessToken(String appKey,String appSecret,String permAuth) {
		String result = "";
		String requestUrl = OPEN_HOST + "/cgi-bin/token?grant_type=client_credential&appKey="+appKey+"&appSecret="+appSecret+"&permAuth="+permAuth;
		try {
			result = HttpClientFactory.get(requestUrl, null);
			logger.info("获取accessToken:{}",result);
		} catch (Exception e) {
			logger.error("error url:{},e:{}",requestUrl,e);
			result = e.getMessage();
		}
		return result;
	}
	
	public static String getDepartmentList(String accessToken,String params) {
		String result = "";
		String requestUrl = OPEN_HOST + "/cgi-bin/department/list?access_token="+accessToken;
		try {
			if(StringUtils.isEmpty(accessToken)) {
				throw new RuntimeException("accessToken为空");
			}
			
			Map<String,String> headers = new HashMap<String,String>();
			headers.put("Content-Type", "application/json");
			result = HttpClientFactory.postBody(requestUrl, params, headers);
			logger.info("获取getDepartmentList:{}",result);
		} catch (Exception e) {
			logger.error("error url:{},e:{}",requestUrl,e);
			result = e.getMessage();
		}
		return result;
	}
	
	public static String getContactUserList(String accessToken,String params) {
		String result = "";
		String requestUrl = OPEN_HOST + "/cgi-bin/contact/simplelist?access_token="+accessToken;
		try {
			if(StringUtils.isEmpty(accessToken)) {
				throw new RuntimeException("accessToken为空");
			}
			
			Map<String,String> headers = new HashMap<String,String>();
			headers.put("Content-Type", "application/json");
			result = HttpClientFactory.postBody(requestUrl, params, headers);
			logger.info("获取getContactUserList:{}",result);
		} catch (Exception e) {
			logger.error("error url:{},e:{}",requestUrl,e);
			result = e.getMessage();
		}
		return result;
	}
	
	public static String sendMsg(String accessToken,String params) {
		String result = "";
		String requestUrl = OPEN_HOST + "/cgi-bin/appmsg/send?access_token="+accessToken;
		try {
			if(StringUtils.isEmpty(accessToken)) {
				throw new RuntimeException("accessToken为空");
			}
			
			Map<String,String> headers = new HashMap<String,String>();
			headers.put("Content-Type", "application/json");
			result = HttpClientFactory.postBody(requestUrl, params, headers);
			logger.info("获取sendMsg:{}",result);
		} catch (Exception e) {
			logger.error("error url:{},e:{}",requestUrl,e);
			result = e.getMessage();
		}
		return result;
	}
	
	public static String getJssdkTicket(String accessToken) {
		String result = "";
		String requestUrl = OPEN_HOST + "/cgi-bin/jssdk/ticket?access_token="+accessToken;
		try {
			if(StringUtils.isEmpty(accessToken)) {
				throw new RuntimeException("accessToken为空");
			}
			result = HttpClientFactory.get(requestUrl, null);
			logger.info("获取jssdkTicket:{}",result);
		} catch (Exception e) {
			logger.error("error url:{},e:{}",requestUrl,e);
			result = e.getMessage();
		}
		return result;
	}

	
}
