package com.mashangban.example.utils;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class CookieUtil {
	private static final Logger logger = LoggerFactory.getLogger(CookieUtil.class);
	
	
	public static final int ONE_DAY = 86400;

	public static String getCookieValue(HttpServletRequest request, String cookieName) {
		return getCookieValue(request,cookieName,"");
	}

	public static String getCookieValue(HttpServletRequest request, String cookieName, String defaultValue) {
		Cookie cookieList[] = request.getCookies();
		if (cookieList == null || cookieName == null)
			return "";
		for (int i = 0; i < cookieList.length; i++) {
			try {
				Cookie cookie = cookieList[i];
				if (cookie.getName().equals(cookieName)){
					return cookie.getValue();
				}
			} catch (Exception e) {
				logger.error("getCookieValue(HttpServletRequest, String, String)", e); //$NON-NLS-1$
			}
		}
		return "";
	}

	public static void setCookieMaxAge(HttpServletRequest request, HttpServletResponse response, String cookieName, int maxAge) {
		Cookie cookieList[] = request.getCookies();

		for (int i = 0; i < cookieList.length; i++) {
			try {
				if (cookieList[i].getName().equals(cookieName)) {
					cookieList[i].setMaxAge(maxAge);
					// cookieList[i].setDomain(domain);
					cookieList[i].setPath("/");
					response.addCookie(cookieList[i]);
				}
			} catch (Exception e) {
				logger.error("setCookieMaxAge(HttpServletRequest, HttpServletResponse, String, int)", e); //$NON-NLS-1$
			}
		}
	}
	
	public static Cookie addCookie(HttpServletRequest request, HttpServletResponse response, String cookieName, String cookieValue, int expiry) {
		Cookie cookie = new Cookie(cookieName, cookieValue);
		cookie.setPath("/");
		String domain = resolveTopDomain(request.getServerName());
		cookie.setDomain("." + domain);
		cookie.setMaxAge(expiry);
		response.addCookie(cookie);
		return cookie;
	}
	
	public static String resolveTopDomain(String domain) {
		String[] strs = domain.split("\\.");
		if(strs.length < 2){
			domain = strs[0];
		}else if(strs.length == 2){
			domain = strs[0] + "." + strs[1];
		}else{
			domain = domain.substring(domain.indexOf(".") + 1);
		}
		return domain;
	}

}
