package com.mashangban.example.utils;

import java.io.IOException;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Collections;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.net.ssl.SSLContext;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.ParseException;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.LayeredConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.conn.ssl.SSLContexts;
import org.apache.http.conn.ssl.TrustStrategy;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 带连接池的HTTP
 */
public class HttpClientUtil {


	private final static Logger logger = LoggerFactory.getLogger(HttpClientUtil.class);

	private static final int CONNECTION_TIMEOUT = 3000;// 连接超时时间
	private static final int SO_TIMEOUT = 5000;// 等待数据超时时间
	private PoolingHttpClientConnectionManager pool = null;
	//整个连接池的大小
	private int maxConnection = 1000;
	//单个路由连接的最大数
	private int maxConnPerRoute = 100;
	private static final String DEFAULT_CHARSET = "UTF-8";
	private int conntimeout = CONNECTION_TIMEOUT;
	private int sotimeout = SO_TIMEOUT;
	private String reqCharset = DEFAULT_CHARSET;
	private String resCharset = DEFAULT_CHARSET;
	private String agentHeader = "Netease/0.1";
	private boolean sslTrustAll = false; // 是否信任任何ssl证书
	
	public HttpClientUtil() {

	}
	
	public HttpClientUtil(int maxConnection) {
		this.maxConnection = maxConnection;
	}

	public HttpClientUtil(int maxConnection, String charset) {
		this.maxConnection = maxConnection;
		this.reqCharset = charset;
	}

	public HttpClientUtil(int maxConnection, String charset, String resCharset) {
		this.maxConnection = maxConnection;
		this.reqCharset = charset;
		this.resCharset = resCharset;
	}

	public HttpClientUtil(int conntimeout, int sotimeout) {
		this.sotimeout = sotimeout;
		this.conntimeout = conntimeout;
	}

	public HttpClientUtil(int maxConnection, int conntimeout, int sotimeout) {
		this(conntimeout, sotimeout);
		this.maxConnection = maxConnection;
	}
	
	public HttpClientUtil(int maxConnection, int conntimeout, int sotimeout, boolean sslTrustAll) {
		this(conntimeout, sotimeout);
		this.maxConnection = maxConnection;
		this.sslTrustAll = sslTrustAll;
	}

	public HttpClientUtil(int maxConnection, String charset, int conntimeout, int sotimeout) {
		this(conntimeout, sotimeout);
		this.maxConnection = maxConnection;
		this.reqCharset = charset;
	}

	public HttpClientUtil(int maxConnection, int maxConnPerRoute, int conntimeout, int sotimeout,boolean sslTrustAll) {
		this(conntimeout, sotimeout);
		this.maxConnection = maxConnection;
		this.maxConnPerRoute = maxConnPerRoute;
		this.sslTrustAll = sslTrustAll;
	}

	private HttpClient httpClient;
	
	public HttpClient getHttpClient() {
		if (httpClient == null) {
			synchronized (this) {
				if (httpClient == null) {					
					RequestConfig defaultRequestConfig = RequestConfig.custom()
					        .setSocketTimeout(sotimeout)
					        .setConnectTimeout(conntimeout)
					        .setConnectionRequestTimeout(conntimeout).build();
					httpClient = HttpClients.custom().setConnectionManager(pool).setDefaultRequestConfig(defaultRequestConfig).setMaxConnTotal(maxConnection).setMaxConnPerRoute(maxConnPerRoute).build();
				}
			}
		}
		return httpClient;
	}

	public HttpClient getHttpClient(boolean forcenew) {
		if (forcenew) {
			RequestConfig defaultRequestConfig = RequestConfig.custom()
			        .setSocketTimeout(sotimeout)
			        .setConnectTimeout(conntimeout)
			        .setConnectionRequestTimeout(conntimeout).build();
			return HttpClients.custom().setConnectionManager(pool).setDefaultRequestConfig(defaultRequestConfig).build();
		} else {
			return getHttpClient();
		}
	}

	@PreDestroy
	public void destroy() throws Exception {
		logger.info("Http connection pool will destory...");
		if (pool != null) {
			pool.shutdown();
		}
		logger.info("Http connection pool destroyed!");
	}

	@PostConstruct
	public void afterPropertiesSet() throws Exception {
		getPool();
	}

	private PoolingHttpClientConnectionManager getPool() {
		if (pool == null) {
			synchronized (this) {
				if (sslTrustAll) {
					RegistryBuilder<ConnectionSocketFactory> registryBuilder = RegistryBuilder.<ConnectionSocketFactory> create();
					ConnectionSocketFactory plainSF = new PlainConnectionSocketFactory();
					registryBuilder.register("http", plainSF);
					try {
						KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
						// 信任任何链接
						TrustStrategy anyTrustStrategy = new TrustStrategy() {
							@Override
							public boolean isTrusted(X509Certificate[] chain, String authType) throws CertificateException {
								return true;
							}
						};
						SSLContext sslContext = SSLContexts.custom().useTLS().loadTrustMaterial(trustStore, anyTrustStrategy).build();
						LayeredConnectionSocketFactory sslSF = new SSLConnectionSocketFactory(sslContext,
								SSLConnectionSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
						registryBuilder.register("https", sslSF);
					} catch (KeyStoreException | KeyManagementException | NoSuchAlgorithmException e) {
						throw new RuntimeException(e);
					}
					Registry<ConnectionSocketFactory> registry = registryBuilder.build();
					pool = new PoolingHttpClientConnectionManager(registry);
				} else {
					pool = new PoolingHttpClientConnectionManager();
				}

				pool.setMaxTotal(maxConnection);
				pool.setDefaultMaxPerRoute(maxConnection);
			}
		}
		return pool;
	}
	
	/**
	 * 直接返回字符
	 * 
	 * @param url
	 * @return
	 * @throws IOException
	 */
	public String getData(String url, List<NameValuePair> params) throws IOException {
		return fetchData(createGet(url, params, null));
	}

	public String getData(String url) throws IOException {
		List<NameValuePair> params = Collections.emptyList();
		return fetchData(createGet(url, params, null));
	}

	public String getData(String url, ApplicationType type) throws IOException {
		List<NameValuePair> params = Collections.emptyList();
		return fetchData(createGet(url, params, type));
	}

	public String getData(String url, List<NameValuePair> params, ApplicationType type) throws IOException {
		return fetchData(createGet(url, params, type));
	}

	public String putData(String url, List<NameValuePair> params) throws IOException {
		return fetchData(createPut(url, params, null));
	}

	public String putData(String url, List<NameValuePair> params, ApplicationType type) throws IOException {
		return fetchData(createPut(url, params, type));
	}

	public String deleteData(String url, List<NameValuePair> params) throws IOException {
		return fetchData(createDelete(url, params, null));
	}

	public String deleteData(String url, List<NameValuePair> params, ApplicationType type) throws IOException {
		return fetchData(createDelete(url, params, type));
	}

	public String postData(String url, final HttpEntity entity, ApplicationType type) throws IOException {
		return fetchData(this.createPost(url, entity, type));
	}

	public String postData(String url, final HttpEntity entity) throws IOException {
		return fetchData(this.createPost(url, entity, null));
	}
	
	public String postData(String url, final List<NameValuePair> params) throws IOException {
		return fetchData(this.createPost(url, params, null));
	}
	
	public String postData(String url, final String params) throws IOException {
		return fetchData(this.createPost(url, params, null));
	}
	
	public String postData(String url, final List<NameValuePair> params,final List<NameValuePair> headers) throws IOException {
		return fetchData(this.createPost(url, params, headers,null));
	}

	public String postData(String url, final List<NameValuePair> params, ApplicationType type) throws IOException {
		return fetchData(this.createPost(url, params, type));
	}

	public String fetchData(HttpRequestBase request) {
		String result = null;
		if (request == null)
			return result;
		HttpClient client = null;
		long watch = System.nanoTime();
		HttpResponse response = null;
		try {
			client = decHttpClient(request);
			response = client.execute(request);
			HttpEntity rsentity = response.getEntity();
			if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
				Charset rescharset = ContentType.getOrDefault(rsentity).getCharset();
				if (rescharset != null && rescharset.name().equals(StandardCharsets.ISO_8859_1)) {
					result = EntityUtils.toString(rsentity);
					if (resCharset == null) {
						result = new String(result.getBytes(rescharset), DEFAULT_CHARSET);
					} else {
						result = new String(result.getBytes(rescharset), resCharset);
					}
				} else {
					if (resCharset != null) {
						result = EntityUtils.toString(rsentity, resCharset);
					} else {
						result = EntityUtils.toString(rsentity);
					}
				}
				if (logger.isDebugEnabled()) {
					logger.debug(" fetch request {} ", result);
				}
			} else {
				logger.error("fetch request return error status:{} code:{}", request.getURI(), response.getStatusLine().getStatusCode());
				request.abort();
			}
			// EntityUtils.consume(rsentity);
		} catch (ClientProtocolException e) {
			logger.error("fetch request error {} msg {}", request.getURI(), e.getMessage());
			request.abort();
		} catch (ParseException e) {
			logger.error("fetch request error {} msg {}", request.getURI(), e.getMessage());
			request.abort();
		} catch (IOException e) {
			logger.error("fetch request error {} msg {}", request.getURI(), e.getMessage());
			request.abort();
		} finally {
			if (response != null) {
				EntityUtils.consumeQuietly(response.getEntity());
			}
			request.releaseConnection();
			if (logger.isDebugEnabled())
				logger.debug("fetch url:{} consume:{} ", request.getURI(), (System.nanoTime() - watch) / 1000);
		}
		return result;
	}

	private HttpClient decHttpClient(HttpRequestBase request) {
		HttpClient client;
		client = getHttpClient();
		request.addHeader("User-Agent", agentHeader);
		return client;
	}

	public static void addHeader(HttpRequestBase request, String key, String val) {
		request.addHeader(key, val);
	}

	/**
	 * 创建post请求
	 * 
	 * @param path
	 *            路径
	 * @return 请求
	 * @throws UnsupportedEncodingException
	 */
	public HttpPost createPost(String url, final List<NameValuePair> params, ApplicationType type) throws UnsupportedEncodingException {
		UrlEncodedFormEntity entity = new UrlEncodedFormEntity(params, reqCharset);
		return createPost(url, entity, type);
	}
	
	/**
	 * @param url
	 * @param params
	 * @param type
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	public HttpPost createPost(String url, final String params, ApplicationType type) throws UnsupportedEncodingException {
		StringEntity entity = new StringEntity(params, ContentType.APPLICATION_JSON);
		return createPost(url, entity, type);
	}
	
	/**
	 * 创建post请求
	 * 
	 * @param path
	 *            路径
	 * @return 请求
	 * @throws UnsupportedEncodingException
	 */
	public HttpPost createPost(String url, final List<NameValuePair> params,final List<NameValuePair> headers, ApplicationType type) throws UnsupportedEncodingException {
		UrlEncodedFormEntity entity = new UrlEncodedFormEntity(params, reqCharset);
		return createPost(url, entity,headers, type);
	}

	/**
	 * 创建URLENCODE
	 * 
	 * @param url
	 * @param params
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	public HttpPost createPost(String url, final List<NameValuePair> params) throws UnsupportedEncodingException {
		return createPost(url, params, null);
	}

	/**
	 * 创建自定义的CONTENT TYPE的内容
	 * 
	 * @param url
	 * @param entity
	 *            StringEntity
	 * @return
	 */
	public HttpPost createPost(String url, HttpEntity entity) {
		return createPost(url, entity, null);
	}

	/**
	 * 创建自定义的CONTENT TYPE的内容 指定需要的返回类型
	 * 
	 * @param url
	 * @param entity
	 * @param accept
	 * @return
	 */
	public HttpPost createPost(String url, HttpEntity entity, ApplicationType accept) {
		HttpPost method = new HttpPost(url);
		if (null != accept) {
			method.addHeader("accept", accept.val());
		}
		method.setEntity(entity);
		return method;
	}
	
	
	/**
	 * 创建自定义的CONTENT TYPE的内容 指定需要的返回类型
	 * 
	 * @param url
	 * @param entity
	 * @param accept
	 * @return
	 */
	public HttpPost createPost(String url, HttpEntity entity,final List<NameValuePair> headers, ApplicationType accept) {
		HttpPost method = new HttpPost(url);
		if (null != accept) {
			method.addHeader("accept", accept.val());
		}
		for(NameValuePair pair:headers){
			method.addHeader(pair.getName(), pair.getValue());
		}
		method.setEntity(entity);
		return method;
	}

	/**
	 * 
	 * @param url
	 * @param params
	 * @return
	 * @throws IOException
	 */
	public HttpGet createGet(String url, final List<NameValuePair> params) throws IOException {
		return createGet(url, params, null);
	}

	/**
	 * 
	 * @param url
	 * @param params
	 * @param accept
	 * @return
	 * @throws IOException
	 */
	public HttpGet createGet(String url, final List<NameValuePair> params, ApplicationType accept) throws IOException {
		HttpGet method = new HttpGet(urlEncode(url, params));
		if (null != accept) {
			method.addHeader("accept", accept.val());
		}
		return method;
	}

	private String urlEncode(String url, final List<NameValuePair> params) {
		if (params == null)
			return url;
		String param = URLEncodedUtils.format(params, reqCharset);
		if (url.indexOf("?") == -1) {
			url += "?" + param;
		} else {
			url += param;
		}
		return url;
	}

	public HttpPut createPut(String url, final List<NameValuePair> params, ApplicationType accept) throws IOException {
		HttpPut method = new HttpPut(url);
		if (params != null) {
			UrlEncodedFormEntity entity = new UrlEncodedFormEntity(params, reqCharset);
			method.setEntity(entity);
		}
		if (null != accept) {
			method.addHeader("accept", accept.val());
		}
		return method;
	}

	public HttpPut createPut(String url, final List<NameValuePair> params) throws IOException {
		return createPut(url, params, null);
	}

	public HttpDelete createDelete(String url, final List<NameValuePair> params) throws IOException {
		return createDelete(url, params, null);
	}

	public HttpDelete createDelete(String url, final List<NameValuePair> params, ApplicationType accept) throws IOException {
		HttpDelete method = new HttpDelete(urlEncode(url, params));
		if (null != accept) {
			method.addHeader("accept", accept.val());
		}
		return method;
	}

	public String getAgentHeader() {
		return agentHeader;
	}

	public void setAgentHeader(String agentHeader) {
		this.agentHeader = agentHeader;
	}
	
	
	public enum ApplicationType {

		JSON("application/json"), XML("application/xml"), TEXT("text/xml"), FORM("application/x-www-form-urlencoded");

		private String type;

		private ApplicationType(String type) {
			this.type = type;
		}

		public String val() {
			return type;
		}
	}
	
}



